import BN = require('bn.js');
import { Decimal } from 'decimal.js';
import { Operation } from '../operation';
import { Address } from '../types';
import { Scheme } from '../scheme';
export declare class CL4RScheme {
    scheme: Scheme;
    constructor(scheme: Scheme);
    /**
     * getRepuationRewardForBatch
     * according to the formula
     * repPerBatch = _repRewardConstA * ((_repRewardConstB/1000) ** batchIndex)
     * @param  repRewardConstA
     * @param  repRewardConstB
     * @param  batchIndex the batchIndex to calculate
     * @return     RepuationRewardForBatch
     */
    getRepuationRewardForBatch(repRewardConstA: string, repRewardConstB: string, batchIndex: number): Promise<Decimal>;
    getAgreementHash(): Promise<string>;
    getReputationRewardForLockingIds(lockingIds: number[], batchIndex: number, repuationRewardForPeriod: Decimal): Promise<Decimal>;
    lock(amount: BN, period: number, batchIndexToLockIn: number, agreementHash: string): Operation<any>;
    extendLocking(extendPeriod: number, batchIndexToLockIn: number, lockingId: number, agreementHash: string): Operation<any>;
    release(beneficiary: Address, lockingId: number): Operation<any>;
    /**
     * redeem redeem reputation for a beneficiary for all is lockingIds
     * @param  beneficiary
     * @param  lockingIds
     * @param  ct4rRedeemerAddress ct4rRedeemerAddress is a optional.
     * @return transaction promise
     */
    redeem(beneficiary: Address, lockingIds: number[], ct4rRedeemerAddress?: Address): Operation<any>;
    getContract(): Promise<any>;
    getContractAddress(): Promise<string>;
    getCT4RRedeemer(ct4rRedeemerAddress?: Address): Promise<any>;
    getScheme(): Scheme;
}
